# defaults.tcl - set default X resources and bindings
#
# Copyright (C) 1994 Allan Brighton (allan@piano.sta.sub.org)


# setup default X resources
# set up some default resources for color, mono or unknown (PC with Windows?)

proc set_default_resources {} {
    switch [winfo screenvisual .] {
	{staticgray} {
	    # Mono 
	    set normalbg white
	    set normalfg black
	    set accentbg black
	    set accentfg white
	    set workspbg white
	    set workspfg black
	    set darkbg white
	    set darkfg white
	}
	{unknown} {
	    # PC with Windows ?
	    set normalbg #c0c0c0
	    set normalfg black
	    set accentbg maroon
	    set accentfg white
	    set workspbg #c0c0c0
	    set workspfg black
	    set darkbg #808080
	    set darkfg #a0a0a0
	}
	{default} {
	    # color or greyscale ?
	    catch {tk colormodel . color}
	    set normalbg #b5c4a2
	    set normalfg black
	    set accentbg maroon
	    set accentfg white 
	    set workspbg #daedc3
	    set workspfg black 
	    set darkbg #7a846e
	    set darkfg #cee0ba
	    option add *activeBackground #cee0ba
	    option add *activeForeground black 
	}
    }

    # add the options to the option database, but allow
    # them to be overridden by the user's X defaults

    option add *background $normalbg
    option add *foreground $normalfg
    option add *accentBackground $accentbg
    option add *accentForeground $accentbg
    option add *workspaceBackground $workspbg
    option add *workspaceForeground $workspfg
    option add *darkBackground $darkbg
    option add *darkForeground $darkfg
    option add *normalFont -adobe-courier-medium-r-*-*-*-120-*-*-*-*-*-*
    option add *boldFont -adobe-courier-bold-r-*-*-*-120-*-*-*-*-*-*
    option add *labelFont -adobe-helvetica-bold-r-*-*-*-120-*-*-*-*-*-*
    
    # Now get the user's X defaults and use them to set specific 
    # widget properties

    set normal_font [option get . normalFont Tk]
    set bold_font [option get . boldFont Tk]
    set label_font [option get . labelFont Tk]
    set normalbg [option get . background Tk]
    set normalfg [option get . foreground Tk]
    set accentbg [option get . accentBackground Tk]
    set accentfg [option get . accentForeground Tk]
    set workspbg [option get . workspaceBackground Tk]
    set workspfg [option get . workspaceForeground Tk]
    set darkbg   [option get . darkBackground Tk]
    set darkfg   [option get . darkForeground Tk]

    option add *Scrollbar.Background $darkbg 
    option add *Scrollbar.Foreground $darkfg 
    option add *Canvas.background $workspbg 
    option add *Canvas.foreground $workspfg 
    option add *Listbox.background $workspbg 
    option add *Listbox.foreground $workspfg 
    option add *Entry.relief sunken
    option add *Scrollbar.relief sunken
    option add *Radiobutton.relief flat
    option add *Checkbutton.relief flat
    option add *Font $normal_font
    option add *Label.font $label_font
    option add *Listbox.font $normal_font
    option add *Checkbutton.font $label_font
}


